/*
 * @(#)file      HelpFrame.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.4
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.easymanager;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Frame;
import java.awt.Panel;
import java.awt.Button;
import java.awt.Dimension;
import java.awt.TextArea;
import java.awt.BorderLayout;
import java.awt.Rectangle;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;

class HelpFrame extends Frame implements ActionListener
{
    public TextArea text = new TextArea(38, 75);
    public Button b = new Button("close");
    private int x = 0;
    private int y = 0;
    public HelpFrame(String name) {
	super(name);
	printHelp();
	this.setLayout(new BorderLayout());
	text.setEditable(false);
	add(text, BorderLayout.NORTH);
	b.addActionListener(this);
	add(b, BorderLayout.SOUTH);
	GraphicsConfiguration gc = 
	    GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration() ;
	Rectangle bounds = gc.getBounds();
	x = 350 + bounds.x;
	y = bounds.y;
	setLocation(x, y);
	setSize(350, 350);
	pack();
    }
    
    public void display() {
	setLocation(x, y);
	show();
    }
    private void printHelp() {
	text.append("***[NEW]****SNMP Graphical MIB discovery****\n");
	text.append("discovermib <-p profile> <-start oid start> <-stop oid stop>\n");
	text.append("If no profile is provided, a standard v1 profile is used\n");
	text.append("If no oid start is provided, 1.1 is used\n");
	text.append("If no oid stop is provided, 2.1 is used\n");
	text.append("resync button to resync, close button to close\n\n");
	
	text.append("****get like request grammar****\n");
	text.append(" <get | getnext | getbulk> : <version> <security params> :< <OID1>,<OID2>, ...,<OIDn> >\n");
	text.append(" Default SNMP version is SNMP V2\n\n");

	text.append("****security parameters****\n");
	text.append(" V1/V2 ==> -r <community> -w <community>\n");
	text.append(" V3    ==> -u <user> -f <noAuthNoPriv | authNoPriv | authPriv> -c <contextname> \n\n");

	text.append("****security parameters profiles****\n");
	text.append("to simplify security parameters you can associate a name to a set of parameters\n");
	text.append("By default 3 profiles are loaded :\n");
	text.append("JDMK_STD_V1_profile : 1 -r public -w private\n");
	text.append("JDMK_STD_V2_profile : 2 -r public -w private\n");
	text.append("JDMK_STD_V3_profile : 3 -u defaultUser -f noAuthNoPriv\n");
	text.append("profile grammar :\n");
	text.append(" <get | getnext | getbulk> :-p profileName:<....>\n\n");
	text.append("****to add, remove and print a profile****\n");
	text.append("Adding twice the same profile is equal to a profile update\n");
	text.append("'addprofile <profile name> <set of params>'\n");
	text.append("'addprofile myV1profile 1 -r public -w private'\n");
	text.append("'addprofile myV3profile 3 -u defaultUser -f authNoPriv'\n");
	text.append("'remprofile <profile name>'\n");
	text.append("'remprofile myV1profile'\n");
	text.append("'printprofile myV1profile'\n");
	text.append("'printallprofile'\n\n");
	
	text.append("****get examples****\n");
	text.append("\t'get:1:<1.3.6.9.0>' V1\n");
	text.append("\t'get:1 -r public -w private:<1.3.6.9.0>' V1\n");
	text.append("\t'get:<1.3.6.9.0,sysDescr.0>' V2\n");
	text.append("\t'get:2 -r public -w private:<1.3.6.9.0>' V2\n");
	text.append("\t'get:3:<1.3.6.9.0,sysDescr.0>' V3\n");
	text.append("\t'get:3 -u defaultUser -f authNoPriv:<1.3.6.9.0,sysDescr.0>' V3\n\n");
	text.append("****getnext examples****\n");
	text.append("\t'getnext:1:<1.3.6.9.0,1.2.1.1.7.0>' V1\n");
	text.append("\t'getnext:<1.3.6.9.0,1.2.1.1.7.0>' V2\n\n");

	text.append("****getbulk example****\n");
	text.append("\t'getbulk,0,5:<1.3.6.9.0,1.2.1.1.7.0>'\n\n");

	text.append("****set request grammar****\n");
	text.append(" set: <version> <security params> :< <<OID1>,BER flag,<value1>>,<<OID1>,BER flag,<value2>>,\n");
	text.append(" ...<<OID1>,BER flag,<value2>> >\n\n");

	text.append("****set examples****\n");
	text.append("\t'set:1:<<1.3.6.9.0,4,Hello>,<1.2.1.1.7.0,2,89>>'\n");
	text.append("\t'set:<<1.3.6.9.0,4,Hello>,<1.2.1.1.7.0,2,89>>'\n\n");

	text.append("****supported BER data type for SNMP set****\n");
	text.append("\t * boolean :               1\n");
	text.append("\t * integer :               2\n");
	text.append("\t * octet string :          4\n");
	text.append("\t * null :                  5\n");
	text.append("\t * object identifier :     6\n");
	text.append("\t * binary string (ox...) : 7\n\n");

	text.append("****trap and inform handling****\n");
	text.append("'trap' to receive traps and informs\n");
	text.append("'untrap' to stop traps and informs\n\n");

	text.append("**** MIB handling****\n");	
	text.append("WARNING : By Default, all oid tables are loaded.\n");
	text.append("Search based on your classpath\n");
	text.append("'loadmib <MIBOidTable class>' (eg :com.sun.nhas.ma.snmp.NHAS_STATISTICS_MIBOidTable) \n");
	text.append("to load a MIB symbols. The MIB table must be in your classpath. Once the mib loaded \n");
	text.append("you can use the symbole when making requests. MIB II symbols are loaded by default. \n");
	text.append("'printmib' to display MIB symbols\n\n");

	text.append("****SNMP V3 connect****\n");
	text.append("'v3connect' to process Usm discovery. Not mandatory. First V3 request will connect.\n\n");
	
	text.append("****Exit easymanager****\n");
	text.append("'exit' to exit\n\n");     
	
    }

    public void actionPerformed(ActionEvent e) {
	hide();
    }
}
