/*
 * @(#)file      MetaFinder.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.6
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.easymanager;

import java.net.InetAddress;
import java.net.URL;
import java.net.URLClassLoader;
import java.io.File;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipEntry;
import java.util.Enumeration;

import com.sun.management.snmp.SnmpOidDatabaseSupport;
import com.sun.management.snmp.SnmpOidRecord;
import com.sun.management.snmp.SnmpOidTable;

public class MetaFinder extends Thread {
    Vector classPathSegmentsDone = new Vector();
    public Vector entries = new Vector();
    static final String CLASS_TAG = ".class";
    URLClassLoader urlClassLoader = null;
    SnmpOidDatabaseSupport oidDatabase = null;
    Class oidTableClass = null;
    public MetaFinder(SnmpOidDatabaseSupport oidDatabase,
		      SnmpOidTable[] tables) {
	this.oidDatabase = oidDatabase;
	if(tables != null) {
	    for(int i = 0; i < tables.length; i++) {
		oidDatabase.add(tables[i]);
		for(Enumeration e = tables[i].getAllEntries().elements();
		    e.hasMoreElements() ;) {
		    SnmpOidRecord rec = (SnmpOidRecord) e.nextElement();
		    if(rec.getType().equals("EN"))
			entries.add(rec);
		}
	    }
	}
	
	try {
	    oidTableClass = Class.forName("com.sun.management.snmp.SnmpOidTable");
	}catch(Exception e) {
	    java.lang.System.out.println(e.getMessage());
	}
    }
    
    public MetaFinder(SnmpOidDatabaseSupport oidDatabase,
		      Iterator it) {
	this.oidDatabase = oidDatabase;
	if(it != null) {
	    while(it.hasNext()) {
		SnmpOidTable table = (SnmpOidTable) it.next();
		oidDatabase.add(table);
		System.out.println("FOUND TABLE : " + table);
		for(Enumeration e = table.getAllEntries().elements();
		    e.hasMoreElements() ;) {
		    SnmpOidRecord rec = (SnmpOidRecord) e.nextElement();
		    if(rec.getType().equals("EN"))
			entries.add(rec);
		}
	    }
	}
	
	try {
	    oidTableClass = Class.forName("com.sun.management.snmp.SnmpOidTable");
	}catch(Exception e) {
	    java.lang.System.out.println(e.getMessage());
	}
    }

    public void run() {
	//java.lang.System.out.println("[METAFINDER]Starting oid table search...");
	find();
	//java.lang.System.out.println("[METAFINDER]Oid table search finished.");
    }
    
    boolean alreadyProcessed(String classPathSegment) {
	for (int i = 0; i < classPathSegmentsDone.size(); i++)
	    if (classPathSegment.equals((String)classPathSegmentsDone.elementAt(i)))
		return true;
	return false;
    }
    
    public void find() {
	try {
	    String classpath = System.getProperty("java.class.path");
	    urlClassLoader = new URLClassLoader(split(classpath));
	    StringTokenizer tokens = new StringTokenizer(classpath,File.pathSeparator);
	    String currSegment;		
	    File currFile;
	    while (tokens.hasMoreTokens()) {
		currSegment = tokens.nextToken();
		if (!alreadyProcessed(currSegment)) {
		    currFile = new File(currSegment);
		    if (currFile.exists()) {
			
			if (currFile.isDirectory()) {
			    loadDirectory(new StringBuffer(), currFile);
			}
			else {
			    loadJar(currFile);
			}
		    }
		}
		classPathSegmentsDone.addElement(currSegment);
	    }
	}catch(Exception e) {}
    }
    
    private void add(String s, Vector v) {	
	String end = "";
	if (s.length() != 0) {	 
	    File f = new File(s);
	    if (f.isDirectory()) {
		end = "/";
	    }
	    try {
		v.addElement(new URL("file:" + s + end));		
	    }	
	    catch(Exception ex) {
		System.out.println("Internal error occurred :"+ex.toString());
		ex.printStackTrace();
	    }
	}
    }
    
    private URL[] split(String s) {
	char pathCh = File.pathSeparatorChar;
	Vector v = new Vector();
	int start = 0;
	for (int i = s.indexOf(pathCh); i != -1; i = s.indexOf(pathCh, start)) {
	    add(s.substring(start, i), v);
	    start = i + 1;
	}
	if (start != s.length())
	add(s.substring(start), v);
	URL[] path = new URL[v.size()];
	v.copyInto(path);
	return path;
    }
    
    private void loadDirectory(StringBuffer buff,
			       File f) {
	try {
	    File f2;
	    String list[] = f.list();
	    for (int i = 0; i < list.length; i++) {
		f2 = new File(f.getPath()+File.separator+list[i]);
		if (f2.isDirectory()) {
		    StringBuffer buff2 = new StringBuffer(buff.toString());
		    buff2.append(list[i] + ".");
		    loadDirectory(buff2, f2);
		}
		else {
		    if (list[i].endsWith(CLASS_TAG)) {
			handleClass(buff.toString() + 
				    list[i].substring(0,list[i].indexOf(CLASS_TAG)));
		    }							       
		}
	    }
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }
    
    private void loadJar(File f) {
	try {
	    ZipInputStream zipFile = new ZipInputStream(new BufferedInputStream(new FileInputStream(f)));
	    ZipEntry file;
	    //------
	    while ((file = zipFile.getNextEntry())!=null) {
		if ((file.isDirectory())||
		    (file.getName().charAt(file.getName().length()-1)=='/')) {					
		    // skip it - we'll parse the dir info from the filename
		}
		else {
		    if (file.getName().endsWith(CLASS_TAG)) {
			StringTokenizer tokens = new StringTokenizer(file.getName().substring(0,file.getName().lastIndexOf(CLASS_TAG)),"/");
			ArrayList path = new ArrayList(tokens.countTokens());
			StringBuffer buff = new StringBuffer();
			while(tokens.hasMoreTokens()) {
			    buff.append(tokens.nextToken());
			    if(tokens.countTokens() != 0)
				buff.append(".");
			}
			handleClass(buff.toString());
		    }
		}
		zipFile.closeEntry();
	    }	
	    zipFile.close();
	    //------
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }   
    
    void handleClass(String str) {
	Class cl = null;
	try {	
	    cl = urlClassLoader.loadClass(str);
	    if(oidTableClass.isAssignableFrom(cl)) {
		SnmpOidTable t = null;
		try {
		    t = (SnmpOidTable) cl.newInstance();
		    if(!str.equals("com.sun.management.snmp.easymanager.RFC1213_MIBOidTable") && !str.equals("com.sun.management.snmp.easymanager.CustomJVM_MANAGEMENT_MIBOidTable") && !str.equals("com.sun.management.snmp.easymanager.CustomRFC1213_MIBOidTable") && !str.equals("com.sun.management.snmp.easymanager.JVM_MANAGEMENT_MIBOidTable") && !str.equals("com.sun.management.snmp.usm.usmmib.SNMP_USER_BASED_SM_MIBOidTable") && !str.equals("com.sun.management.snmp.SnmpOidDatabaseSupport")) {
			java.lang.System.out.println("[METAFINDER] " + str);
		    }
		    
		    oidDatabase.add(t);
		    //Sort all the entries in order to define table meta
		    for(Enumeration e = t.getAllEntries().elements();
			e.hasMoreElements() ;) {
			SnmpOidRecord rec = (SnmpOidRecord) e.nextElement();
			if(rec.getType().equals("EN"))
			    entries.add(rec);
		    }
		    
		}catch(Exception e) {
		    //java.lang.System.out.println("Can't load class" + e.getMessage());
		}
		
	    }
	}catch(Throwable e) {
	    //java.lang.System.out.println("Can't load class" + e.getMessage());
	}
    }
}
