/*
 * @(#)file      SnmpJdmkNotificationLcd.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.5
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.internal.notification;


import java.lang.reflect.Array;
import java.util.Enumeration;
import java.util.Vector;

import com.sun.jdmk.trace.Trace;
import com.sun.management.snmp.rfc2573.notification.SnmpNotificationLcd;
import com.sun.management.snmp.rfc2573.notification.SnmpNotificationData;
import com.sun.management.snmp.rfc2573.target.SnmpTargetLcd;
import com.sun.management.snmp.rfc2573.target.SnmpTargetData;

import com.sun.management.snmp.SnmpEngine;

public class SnmpJdmkNotificationLcd implements SnmpNotificationLcd {

    SnmpNotificationReader notifReader = null;
    String notifFile = null;
    SnmpTargetLcd targetsLcd;
    boolean init = false;
    SnmpEngine engine = null;

    /**
     * Create the local configuration data
     * @param targetsLcd target local configuration
     * @param notifFile persistent file
     */
    public SnmpJdmkNotificationLcd(SnmpEngine engine,
				   SnmpTargetLcd targetsLcd,
				   String notifFile) {

	this.notifFile = notifFile;
	this.targetsLcd = targetsLcd;
	this.engine = engine;
    }

    /**
     * addNotification Add the notification in Lcd
     * which is a set of data and in the persistent file
     */

    public void addNotification(String notifyName,
				String notifyTag,
				int notifyType,
				int storageType) throws IllegalArgumentException {

	notifReader.addNotification(notifyName,
				    notifyTag,
				    notifyType,
				    storageType);
    }
    
    public void removeNotification(String notifyName) throws IllegalArgumentException {
	notifReader.removeNotification(notifyName);
    }
    
    public Enumeration getNotifications() {
	return notifReader.getNotifications();
    }
   
    public SnmpNotificationData getNotification(String name) {
	return notifReader.getNotification(name);
    } 

    public void readConfiguration() {
	if(init) return;
	notifReader = new SnmpNotificationReader(notifFile);
	init = true;
    }

    public Enumeration getTargets(String notifTag) {


	if (notifTag.length()==0) return null;

	Vector selectedTargets = new Vector();
	Enumeration targets = targetsLcd.getTargets();

        while (targets.hasMoreElements()) {
	    SnmpTargetData data = (SnmpTargetData) targets.nextElement();

	    if (data.tags.indexOf(notifTag)>=0) {
		selectedTargets.add(data);
	    }    
	}

	return selectedTargets.elements();
    }

   boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
	if (isTraceOn()) {
	    Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
	}
    }

    void trace(String func, String info) {
	if (isTraceOn()) {
	    trace(dbgTag, func, info);
	}
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
	if (isDebugOn()) {
	    Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
	}
    }

    void debug(String clz, String func, Throwable exception) {
	if (isDebugOn()) {
	    Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
	}
    }

    void debug(String func, String info) {
	if (isDebugOn()) {
	    debug(dbgTag, func, info);
	}
    }
    
    void debug(String func, Throwable exception) {
	if (isDebugOn()) {
	    debug(dbgTag, func, exception);
	}
    }
    
    String dbgTag = "SnmpJdmkNotificationLcd"; 

}
