/*
 * @(#)file      SnmpNotificationReader.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.5
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.internal.notification;

import java.util.Enumeration;

import com.sun.jdmk.trace.Trace;

import com.sun.management.snmp.rfc2573.internal.SnmpPersistRowFileReader;
import com.sun.management.snmp.rfc2573.notification.SnmpNotificationData;
import com.sun.management.snmp.rfc2573.target.SnmpTargetData;

import com.sun.management.internal.snmp.SnmpPersistRowFileConsumer;

import com.sun.management.snmp.SnmpEngine;

class NotificationConsumer implements SnmpPersistRowFileConsumer {
 
    static final int NOTIFY_NAME = 0;
    static final int NOTIFY_TAG = 1;
    static final int NOTIFY_TYPE = 2;
    static final int NOTIFY_STORAGETYPE = 3;

    SnmpNotificationData notif = new SnmpNotificationData();
    SnmpNotificationReader reader = null;

    NotificationConsumer(SnmpNotificationReader reader) {
	this.reader = reader;
    }

    void reset() {
	notif.notifyName = null;
	notif.notifyTag = null;
	notif.notifyType = -1;
	notif.notifyStorage = -1;
    }

    public Object rowBegin(String row, int line) {
	reset();
	return notif;
    }
    
    public void rowEnd(Object rowContext, String row, int line) throws Exception  {

	SnmpNotificationData data = (SnmpNotificationData) rowContext;
	reader.rowRead(data, row, line);
    }
    
    public void treatToken(String row,
			   int lineNumber,
			   String token,
			   int context,
			   Object rowContext,
			   boolean err) throws Exception {


	SnmpNotificationData notif = (SnmpNotificationData) rowContext;

	if( err || (notif == null) ) 
	    throw new Exception("Badly formed notif entry. Line ["+
				lineNumber+"], data [" + row+"]");
	try {
	    switch(context) {
	    case NOTIFY_NAME:
		notif.notifyName = token;
		break;
	    case NOTIFY_TAG:
		notif.notifyTag = token;
		break;
	    case NOTIFY_TYPE:
		notif.notifyType = Integer.parseInt(token);
		break;
	    case NOTIFY_STORAGETYPE:
		notif.notifyStorage = Integer.parseInt(token);
	    default:
	    }
	} 
	catch(NumberFormatException e) {
	    throw new Exception(e.getMessage()+
				" Line :" + 
				lineNumber + 
				".\n" + 
				" Row : [" + 
				row +"]");
	}
    }
}


class SnmpNotificationReader extends SnmpPersistRowFileReader {

    public SnmpNotificationReader(String file) {
	super(file, "notificationEntry");
	init(new NotificationConsumer(this));
    }
  
    public SnmpNotificationData getNotification(String name) {
	return (SnmpNotificationData) getObject(name);
    }

    public Enumeration getNotifications() {
	return getObjects();
    }
    
    void removeNotification(String notifyName) throws IllegalArgumentException {
	 removeObject(notifyName);
	 flushInFile();
    }

    public void addNotification(String notifyName,
				String notifyTag,
				int notifyType,
				int storageType) throws IllegalArgumentException {

	SnmpNotificationData data = new SnmpNotificationData();
	data.notifyName = notifyName;
	data.notifyTag = notifyTag;
	data.notifyType = notifyType;
	data.notifyStorage = storageType;
	addNotification(data);
	flushInFile();
    }

    void addNotification(SnmpNotificationData data) throws  IllegalArgumentException {
	putObject(data.notifyName, data);
    }

    void rowRead(SnmpNotificationData d, String row, int line) throws Exception {
	addNotification((SnmpNotificationData) d.clone());
    }
}
