/*
 * @(#)file      SnmpAgentProxyImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.6
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.internal.v3proxy;

import com.sun.management.snmp.rfc2573.v3proxy.SnmpAgentProxy;
import com.sun.management.snmp.rfc2573.v3proxy.SnmpProxyData;
import com.sun.management.snmp.rfc2573.internal.v3proxy.SnmpProxyFactory;

import com.sun.jdmk.trace.Trace;

import com.sun.management.snmp.agent.SnmpProxy;
import com.sun.management.snmp.agent.SnmpMibRequest;
import com.sun.management.snmp.SnmpEngine;

import com.sun.management.snmp.SnmpStatusException;

public class SnmpAgentProxyImpl implements SnmpAgentProxy {
    SnmpProxyData proxyData = null;
    SnmpEngine engine = null;
    SnmpProxyFactory proxyFactory = null;
    SnmpProxy proxy = null;
    private String dbgTag = "SnmpAgentProxyImpl";
    public SnmpAgentProxyImpl(SnmpProxyData proxyData,
			      SnmpProxyFactory proxyFactory,
			      SnmpEngine engine) {
	this.proxyData = proxyData;
	this.proxyFactory = proxyFactory;
	this.engine = engine;
    }
    
    private void activateProxy() throws Exception {
	if(proxy == null) {
	    proxyData.singleOut.activateNonAuthoritativePeer();
	    proxy = proxyFactory.createProxy(engine, proxyData);
	    
	    if(proxy == null) {
		if(isDebugOn())
		    debug("activateProxy", "WARNING. PROXY NOT CREATED. No proxy found for Message processing model and or security model.");
		throw new IllegalArgumentException("WARNING. PROXY NOT CREATED. No proxy found for Message processing model and or security model.");
	    }
	    else
		if(isDebugOn())
		    debug("activateProxy", "Proxy activated successfully");
	}
    }
    
    public void get(SnmpMibRequest req) throws SnmpStatusException {
	try {
	    activateProxy();
	}catch(Exception e) {
	    
	    if(isDebugOn()) {
		e.printStackTrace();
		debug("get", "Activation failed :" + e);
	    }
	    proxy = null;
	    return;
	}
	try {
	    proxy.get(req);
	}catch(IllegalArgumentException e) {
	    if(isDebugOn())
		debug("get", "Get request failed :" + e);
	    proxyData.singleOut.deactivateNonAuthoritativePeer();
	    proxy = null;
	    return;
	}
    }
    
    public void getNext(SnmpMibRequest req) throws SnmpStatusException {
	try {
	    activateProxy();
	}catch(Exception e) {
	    
	    if(isDebugOn()) {
		e.printStackTrace();
		debug("getNext", "Activation failed :" + e);
		
	    }
	    proxy = null;
	    return;
	}
	try {
	    proxy.getNext(req);
	}catch(IllegalArgumentException e) {
	    if(isDebugOn())
		debug("getNext", "GetNext request failed :" + e);
	    proxyData.singleOut.deactivateNonAuthoritativePeer();
	    proxy = null;
	    return;
	}
    }

    public void getBulk(SnmpMibRequest req,
			int nonRepeat, 
			int maxRepeat) throws SnmpStatusException {
	try {
	    activateProxy();
	}catch(Exception e) {
	    if(isDebugOn())
		debug("getBulk", "Activation failed :" + e);
	    proxy = null;
	    return;
	}
	try {
	    proxy.getBulk(req, nonRepeat, maxRepeat);
	}catch(IllegalArgumentException e) {
	    if(isDebugOn())
		debug("getBulk", "GetBulk request failed :" + e);
	    proxyData.singleOut.deactivateNonAuthoritativePeer();
	    proxy = null;
	    return;
	}
    }
    
    public void set(SnmpMibRequest req) throws SnmpStatusException {
	try {
	    activateProxy();
	}catch(Exception e) {
	    if(isDebugOn())
		debug("set", "Activation failed :" + e);
	    proxy = null;
	    return;
	}
	try {
	    proxy.set(req);
	}catch(IllegalArgumentException e) {
	    if(isDebugOn())
		debug("set", "Set request failed :" + e);
	    proxyData.singleOut.deactivateNonAuthoritativePeer();
	    proxy = null;
	    return;
	}
    }
    private boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_ADAPTOR_SNMP);
    }

    private void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_ADAPTOR_SNMP, clz, func, info);
    }

    private void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    private boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_ADAPTOR_SNMP);
    }

    private void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_ADAPTOR_SNMP, clz, func, info);
    }

    private void debug(String func, String info) {
        debug(dbgTag, func, info);
    }

}
