/*
 * @(#)file      SnmpNotificationHandler.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.11
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.manager.internal.notification;

// Util import
import java.util.Enumeration;

// JMX RI
import com.sun.management.snmp.SnmpInt;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.SnmpNull;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.SnmpValue;
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.manager.SnmpRequest;
import com.sun.management.snmp.manager.SnmpSession;

// jdmk import
import com.sun.jdmk.TraceManager;
import com.sun.jdmk.trace.Trace;

import com.sun.management.snmp.SnmpOidTableSupport;
import com.sun.management.snmp.EnumRowStatus;

// Contrib import
import com.sun.management.snmp.rfc2573.notification.mib.SNMP_NOTIFICATION_MIBOidTable;
import com.sun.management.snmp.rfc2573.manager.internal.notification.SnmpNotificationData;

public class SnmpNotificationHandler  {
        
    public static void setNotif(SnmpSession session,
			 SnmpNotificationData data) throws SnmpStatusException {
		
	SnmpVarBindList list = new SnmpVarBindList();

	//Notif Index
	SnmpString name = new SnmpString(data.notifyName);
	SnmpOid nameOid = name.toOid();
	long length = data.notifyName.length();

	// Notif Tag
	SnmpOid oidTag = new SnmpOid("1.3.6.1.6.3.13.1.1.1.2");
	SnmpString tag = new SnmpString(data.notifyTag);
	oidTag.append(length);
	oidTag.append(nameOid);
	if(isDebugOn())
	    debug("setNotif", oidTag.toString());
	list.addVarBind(new SnmpVarBind(oidTag, tag));
    

	// Notif Type
	SnmpOid oidType = new SnmpOid("1.3.6.1.6.3.13.1.1.1.3");
	SnmpInt type = new SnmpInt(data.notifyType);
	oidType.append(length);
	oidType.append(nameOid);
	if(isDebugOn())
	    debug("setNotif", oidType.toString());
	list.addVarBind(new SnmpVarBind(oidType, type));
	
	// Storage type
	SnmpOid oidStorage = new SnmpOid("1.3.6.1.6.3.13.1.1.1.4");
	SnmpInt storage = new SnmpInt(data.notifyStorage);
	oidStorage.append(length);
	oidStorage.append(nameOid);
	if(isDebugOn())
	    debug("setNotif", oidStorage.toString());
	list.addVarBind(new SnmpVarBind(oidStorage, storage));

	// RowStatus
	SnmpOid oidRowStatus = new SnmpOid("1.3.6.1.6.3.13.1.1.1.5");
	oidRowStatus.append(length);
	oidRowStatus.append(nameOid);
	EnumRowStatus createAndGo = new EnumRowStatus("createAndGo");
	if(isDebugOn())
	    debug("setNotif", oidRowStatus.toString());
	list.addVarBind(new SnmpVarBind(oidRowStatus, 
					createAndGo.toSnmpValue()));
	// Send the SET request
	SnmpRequest request = session.snmpSetRequest(null, list);

	boolean completed = request.waitForCompletion(session.getDefaultPeer().getTimeout());
	if (completed == false) {
	  if (isTraceOn())
	      trace("setNotif", "Request timed out: check reachability of agent.\n");
	  throw new SnmpStatusException(SnmpDefinitions.snmpReqTimeout, 
				0);
	}

	// Check the status
	int errorStatus = request.getErrorStatus();
	if (errorStatus != SnmpDefinitions.snmpRspNoError) {
	    if (isTraceOn()) {
		trace("setNotif","************* Error status = " + SnmpRequest.snmpErrorToString(errorStatus));
		trace("setNotif","Error index = " + request.getErrorIndex());
		throw new SnmpStatusException(errorStatus,
					      request.getErrorIndex());
	    }
	} else {
	    if (isDebugOn()) {
		SnmpVarBindList result = request.getResponseVarBindList();
		debug("setNotif","Result: \n" + result.varBindListToString());
	    }
	}
    }

    public static void removeNotif(SnmpSession session,
			    String notifyIndex) throws SnmpStatusException {
		
	SnmpVarBindList list = new SnmpVarBindList();

	//Name Index
	SnmpString name = new SnmpString(notifyIndex);
	SnmpOid nameOid = name.toOid();
	long length = notifyIndex.length();

	//RowStatus
	SnmpOid oidRowStatus = new SnmpOid("1.3.6.1.6.3.13.1.1.1.5");
	oidRowStatus.append(length);
	oidRowStatus.append(nameOid);
	EnumRowStatus destroy = new EnumRowStatus("destroy");
	if(isDebugOn())
	    debug("removeNotif", oidRowStatus.toString());
	list.addVarBind(new SnmpVarBind(oidRowStatus, 
					destroy.toSnmpValue()));

	// Send the SET request
	SnmpRequest request = session.snmpSetRequest(null, list);

	// Wait for completion
	boolean completed = request.waitForCompletion(session.getDefaultPeer().getTimeout());
	if (completed == false) {
	  if (isTraceOn())
	      trace("removeNotif", "Request timed out: check reachability of agent.\n");
	  throw new SnmpStatusException(SnmpDefinitions.snmpReqTimeout, 
					0);
	}

	// Check the status
	int errorStatus = request.getErrorStatus();
	if (errorStatus != SnmpDefinitions.snmpRspNoError) {
	    if (isTraceOn()) {
		trace("removeNotif","************* Error status = " + SnmpRequest.snmpErrorToString(errorStatus));
		trace("removeNotif","Error index = " + request.getErrorIndex());
		throw new SnmpStatusException(errorStatus,
					      request.getErrorIndex());
	    }
	} else {
	    if (isDebugOn()) {
		SnmpVarBindList result = request.getResponseVarBindList();
		debug("removeNotif","Result: \n" + result.varBindListToString());
	    }
	}
    }

	
    // TRACES & DEBUG
    //---------------
    
    static boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    static void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    static void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    static boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    static void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    static void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }

    static void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    static void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }
    
    static String dbgTag = "SnmpTargetsParamsHandler"; 
}
