package com.sun.management.snmp.rfc2573.notification.mib;

// java imports
//
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;

// jmx imports
//
import javax.management.MBeanServer;
import javax.management.ObjectName;
import com.sun.management.snmp.SnmpCounter;
import com.sun.management.snmp.SnmpCounter64;
import com.sun.management.snmp.SnmpGauge;
import com.sun.management.snmp.SnmpInt;
import com.sun.management.snmp.SnmpUnsignedInt;
import com.sun.management.snmp.SnmpIpAddress;
import com.sun.management.snmp.SnmpTimeticks;
import com.sun.management.snmp.SnmpOpaque;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.SnmpStringFixed;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpNull;
import com.sun.management.snmp.SnmpValue;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpStatusException;

// jdmk imports
//
import com.sun.jdmk.trace.Trace;
import com.sun.management.snmp.agent.SnmpIndex;
import com.sun.management.snmp.agent.SnmpMib;
import com.sun.management.snmp.agent.SnmpMibTable;
import com.sun.management.snmp.agent.SnmpMibSubRequest;
import com.sun.management.snmp.agent.SnmpTableEntryFactory;
import com.sun.management.snmp.agent.SnmpTableCallbackHandler;
import com.sun.management.snmp.agent.SnmpTableSupport;
import com.sun.management.snmp.EnumRowStatus;



//Contrib imports
import com.sun.management.snmp.rfc2573.notification.SnmpNotificationLcd;
import com.sun.management.snmp.rfc2573.notification.SnmpNotificationData;


public class TableSnmpNotifyTableImpl extends TableSnmpNotifyTable {

    boolean init=false;
    SnmpNotificationLcd lcd=null;

    public TableSnmpNotifyTableImpl(SnmpMib myMib,
				    SnmpNotificationLcd lcd) {
	super(myMib);
	init(lcd);
	init=true;
    }

    public TableSnmpNotifyTableImpl(SnmpMib myMib,
				    MBeanServer server,
				    SnmpNotificationLcd lcd) {
	super(myMib, server);
	init(lcd);
	init=true;
    }
 

    public void addEntryCb(int pos, SnmpOid row, ObjectName name, 
			   Object entry, SnmpMibTable meta) throws SnmpStatusException {
	if(isDebugOn()) debug("addEntryCb","");
    }

  
    public void removeEntryCb(int pos, SnmpOid row, ObjectName name, 
			      Object entry, SnmpMibTable meta) throws SnmpStatusException {

	super.removeEntryCb(pos, row, name, entry, meta);


	if(isDebugOn())	debug("removeEntryCb","");

	//Ask the Lcd to remove The user from the conf.
	SnmpNotifyEntry e = (SnmpNotifyEntry)entry;
	lcd.removeNotification(e.SnmpNotifyName);
    }

    public Object createSnmpNotifyEntryMBean(SnmpMibSubRequest req,
						 SnmpOid rowOid, 
						 int depth, 
						 ObjectName entryObjName,
						 SnmpMibTable meta, 
						 String aSnmpNotifyName) throws SnmpStatusException  {

      
        SnmpNotifyEntryImpl entry = new SnmpNotifyEntryImpl(theMib,
							    lcd);
        entry.SnmpNotifyName = aSnmpNotifyName;
        return entry;
    }

    private void init(SnmpNotificationLcd lcd) {

	setCreationEnabled(true);
	this.lcd = lcd;

	for (Enumeration notifs = lcd.getNotifications(); notifs.hasMoreElements();) {

	    SnmpNotificationData data = (SnmpNotificationData) notifs.nextElement();
	    SnmpNotifyEntryImpl entry = null;

	    try {
		entry = new SnmpNotifyEntryImpl(theMib, 
						lcd,
						data);
	    } 
	    catch(Exception e) {
		
		if(isDebugOn()) {
		    debug("init","Entry NOT created. :" + e.getMessage());
		    return;
		}
	    }

	    entry.SnmpNotifyRowStatus = new EnumSnmpNotifyRowStatus(EnumRowStatus.active);
	    if(isDebugOn()) {
		debug("init","Adding :" + data.notifyName);
	    }
	    
	    try {

		//Add the entry in the table.
		addEntry(entry);
		if(isDebugOn()) {
		    debug("init","Added :" + data.notifyName);
		}
		
		if(server != null)
		    entry.addInMBeanServer(server);
		
	    }catch(SnmpStatusException s) {
		if(isDebugOn())
		    debug("init", s);
	    }
	}
    }
 
    // TRACES & DEBUG
    //---------------
    
    boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }

    void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }
    
    String dbgTag = "TableSnmpNotifyTableImpl";    
}
