/*
 * @(#)file      AgentV3.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.17
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */

// java import
//
import java.net.InetAddress;
import java.net.UnknownHostException;
// RI import
//
import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.SnmpScopedPduRequest;
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpNull;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.manager.usm.SnmpUsmPeer;
import com.sun.management.snmp.manager.usm.SnmpUsmParameters;
import com.sun.management.snmp.SnmpEngineId;
import com.sun.management.snmp.SnmpEngine;
import com.sun.management.snmp.SnmpEngineParameters;
import com.sun.management.snmp.SnmpUnknownModelException;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpIpAddress;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.SnmpInt;
import com.sun.management.snmp.SnmpValue;
import com.sun.management.snmp.manager.SnmpPeer;
import com.sun.management.snmp.manager.SnmpSession;
import com.sun.management.snmp.manager.SnmpParameters;
// jdmk import
//
import com.sun.jdmk.TraceManager;
import com.sun.jdmk.trace.Trace;
import com.sun.jdmk.comm.HtmlAdaptorServer;
import com.sun.management.comm.SnmpV3AdaptorServer;
import com.sun.management.comm.SnmpAdaptorServer;

import com.sun.management.snmp.usm.*;
import com.sun.management.snmp.agent.SnmpTrap;
import com.sun.management.comm.SnmpV3InformRequest;

//Contribution imports
import com.sun.management.snmp.rfc2573.manager.SnmpV3AppliMibRegistration;

/**
 * The Agent class provides a simple example on how to use the SNMP protocol adaptor.
 * A subset of MIB II (RFC1213) is implemented. The MIB is loaded and initialized. 
 * As such you can now see the MIB using your favorite SNMP manager, 
 * or you can use a web browser and see the MIB through the HTML adaptor.
 * 
 * When calling the program, you can specify:
 *
 * In this example, the SNMP adaptor is started on port 8105, 
 * and the traps are sent to the port 8086, i.e a non standard ports for SNMP. 
 * As such you do not need to be root to start the agent.
 */
/**
 * This class is used to generate traps.
 **/


public class AgentV3 {
    //Class used to send traps.
    //
    private static class TrapGenerator  extends Thread {
	
	SnmpV3AdaptorServer server = null;
	SnmpVarBindList lgarbage = null;
	
	private void sendV3() {
	    SnmpVarBindList varBindList = new SnmpVarBindList();
	    
	    SnmpOid oid1 = new SnmpOid("1.3.6.1.2.1.2.2.1.1.99");
	    SnmpInt value1 = new SnmpInt(99);
	    SnmpVarBind varBind1 = new SnmpVarBind(oid1, (SnmpValue) value1);
	    
	    varBindList.addVarBind(varBind1);
	    
	    try {
		server.snmpV3UsmTrap("defaultUser",
				     SnmpDefinitions.authNoPriv,
				     "TEST-CONTEXT",
				     new SnmpOid("1.2.3.4.5.6.7.8.9.0"), 
				     varBindList);
		println("V3 TRAP sent\n");
	    }catch(Exception e) {
		e.printStackTrace();
	    }
	}
	
	public void run() {
	    while(true) {
		println("\nType a key in order to launch an SNMP V3 trap.");
		try {
		    java.lang.System.in.read();
		} catch(Exception e) {	
		    e.printStackTrace();
		}
		
		sendV3();
	    }
	}
	
	public TrapGenerator(SnmpAdaptorServer server) {
	    
	    this.server = (SnmpV3AdaptorServer) server;
	
	    //Initialize a list of Varbinds
	    lgarbage = new SnmpVarBindList();
	    lgarbage.addVarBind(new 
		SnmpVarBind(new SnmpOid("1.2.3.4.5.6.0.0"), 
			    new SnmpString("Test values")));
	    lgarbage.addVarBind(new 
		SnmpVarBind(new SnmpOid("1.2.3.4.5.6.1.0"), 
			    new SnmpNull()));
	    lgarbage.addVarBind(new 
		SnmpVarBind(new SnmpOid("1.2.3.4.5.6.2.0"), 
			    new SnmpInt (43)));
	    lgarbage.addVarBind(new 
		SnmpVarBind(new SnmpOid("1.2.3.4.5.6.3.0"), 
			    new SnmpString ("Test values")));
	    
	    //Instanciate an address OID varBind
	    SnmpVarBind vb = null;
	    try{
		final byte[] inet = 
		    InetAddress.getLocalHost().getAddress();
		final SnmpIpAddress addr = new SnmpIpAddress(inet);
		vb = new SnmpVarBind(SnmpTrap.snmpTrapAddressOid,addr);
	    }catch(UnknownHostException e) {
		vb = new SnmpVarBind(SnmpTrap.snmpTrapAddressOid, 
				     new SnmpIpAddress("123.33.33.33")); 
	    }
	    lgarbage.addVarBind(vb);
	}
    }
    
    static SnmpV3AdaptorServer snmpAdaptor = null;

    /**
     * Main entry point.
     * When calling the program, you can specify:
     *
     * In order to get traces from the SNMP adaptor server, invoke the program as follows:
     *  java -DINFO_ADAPTOR_SNMP -Djdmk.security.file=./jdmk.security AgentV3

     */
    public static void main(String args[]) {
        
        MBeanServer server;
        ObjectName htmlObjName;
        ObjectName snmpObjName;
        int htmlPort = 8102;
        int snmpPort = 8105;

       	//Proxy args
	String proxyHost = "localhost";
	String proxyPort = "8085";
	if(args.length >= 1)
	    proxyHost = args[0];
	if(args.length >= 2)
	    proxyPort = args[1];

	
        // Initialize trace property.
        // 
        try {
            TraceManager.parseTraceProperties();
            Trace.send(Trace.LEVEL_TRACE, Trace.INFO_MISC, "AgentV3", "main", "Default trace is on");
        } catch(java.io.IOException e) {
            e.printStackTrace();
        }
    
        try {
            server = MBeanServerFactory.createMBeanServer();
            String domain = server.getDefaultDomain();

            // Create and start the HTML adaptor.
            //
            htmlObjName = new ObjectName(domain + ":class=HtmlAdaptorServer,protocol=html,port=" + htmlPort);
            Trace.send(Trace.LEVEL_TRACE, Trace.INFO_MISC, "Agent", "main", 
                       "Adding HTML adaptor to MBean server with name \n\t" + htmlObjName);
            println("NOTE: HTML adaptor is bound on TCP port " + htmlPort);
            HtmlAdaptorServer htmlAdaptor = new HtmlAdaptorServer(htmlPort);
            server.registerMBean(htmlAdaptor, htmlObjName);
            htmlAdaptor.start();
                  
            //
            // SNMP specific code:
            //
            snmpObjName = new ObjectName(domain + ":class=SnmpV3AdaptorServer,protocol=snmp,port=" + snmpPort);
            Trace.send(Trace.LEVEL_TRACE, Trace.INFO_MISC, "Agent", "main", 
                       "Adding SNMP adaptor to MBean server with name \n\t" + snmpObjName);
            println("NOTE: SNMP Adaptor is bound on UDP port " + snmpPort);
	    SnmpEngineParameters params = new SnmpEngineParameters();
	    params.setEngineId(SnmpEngineId.createEngineId("localhost:" + snmpPort));
            snmpAdaptor = new SnmpV3AdaptorServer(params,
						  null,
						  null,
						  snmpPort,
						  null);
	    // The V3 proxy is listening on 8086.
	    snmpAdaptor.setTrapPort(new Integer(8086));
	    
            server.registerMBean(snmpAdaptor, snmpObjName);
	    
            snmpAdaptor.start();
	    
	    if(snmpAdaptor.getEngine() == null) {
		println("Unable to create the engine, check your conf.");
		Runtime.getRuntime().exit(0);
	    }
	    println("\n***ENGINE Id ["+ 
		    snmpAdaptor.getEngine().getEngineId()+"]********");
	    
            // Create an instance of the customized MIB
            //
            RFC1213_MIB mib2 = new RFC1213_MIB_IMPL();
	    
            // Bind the SNMP adaptor to the MIB in order to make the MIB 
            // accessible through the SNMP protocol adaptor.
            // If this step is not performed, the MIB will still live in 
            // the JDMK agent:
            // its objects will be addressable through HTML but not SNMP.
            //
	    mib2.init();
	    snmpAdaptor.addMib(mib2);
	    snmpAdaptor.addMib(mib2, "TEST-CONTEXT");
	    
	    println("Press Return to register the agent in the proxy [" + proxyHost + ":"+proxyPort+"]");
	    java.lang.System.in.read();
	    
	      //Peer
	    SnmpPeer proxy = new SnmpPeer(proxyHost, 
					  Integer.parseInt(proxyPort));
	    SnmpSession session = new SnmpSession("Register session");
	    session.setDefaultPeer(proxy);
	    SnmpParameters p = new SnmpParameters("public", "private");
	    proxy.setParams(p);
	  
	    //Params registering
	    //agent2_paramsV3,3,3,defaultUser,2,3
	    SnmpV3AppliMibRegistration.registerParams(session,
						      "agent2_paramsV3",
						      3,
						      3,
						      "defaultUser",
						      2);
	    // The incoming matching criteria
	    // Only SMP V3 manager can acces this agent.
	    //agent2_criteriaV3,3,3,defaultUser,2,3
	    SnmpV3AppliMibRegistration.registerParams(session,
						      "agent2_criteriaV3",
						      3,
						      3,
						      "defaultUser",
						      2);
	    //Target registering
	    //agent2_targetV3,snmpUDPDomain,localhost:8105,10000,3,useless list,agent2_paramsV3,3
	    SnmpV3AppliMibRegistration.registerTarget(session,
						      "agent2_targetV3",
						      "1.3.6.1.6.1.1",
						      "localhost/8105",
						      10000,
						      3,
						      "useless list",
						      "agent2_paramsV3");
	   
	    //Proxy registering
	    //proxy_agent2,1,localhost:8105,TEST-CONTEXT,agent2_criteriaV3,agent2_targetV3,useless list,3
	    SnmpV3AppliMibRegistration.registerProxy(session,
						     "proxy_agent2",
						     1,
						     SnmpEngineId.createEngineId("localhost:8105"),
						     "TEST-CONTEXT",
						     "agent2_criteriaV3",
						     "agent2_targetV3",
						     "useless list");
	    
	    println("Agent successfuly registered in proxy");
	    
	    println("Press Return to unregister this agent from the proxy");
	    java.lang.System.in.read();
	    SnmpV3AppliMibRegistration.unregisterParams(session,
							"agent2_paramsV3");
	    SnmpV3AppliMibRegistration.unregisterParams(session,
							"agent2_criteriaV3");
	    SnmpV3AppliMibRegistration.unregisterTarget(session,
							"agent2_targetV3");
	    SnmpV3AppliMibRegistration.unregisterProxy(session,
						       "proxy_agent2");
	    
	    println("Agent successfuly unregistered from proxy");
	    println("Press Return if you want to send traps");
	    java.lang.System.in.read();
	    
	    //Instantiate and start the TrapGenerator.
	    TrapGenerator trapGenerator = new TrapGenerator(snmpAdaptor);
	    trapGenerator.start();
	    
	    println("Press crtl-c in order to kill the agent"); 
	    
	    trapGenerator.join();
            
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    static private void println(String msg) {
	java.lang.System.out.println(msg);
    }
}
