/*
 * @(#)file      JmxMBeanServerMetaImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.5
 * @(#)lastedit  04/04/07
 * 
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java imports
//
import java.io.Serializable;

// jmx imports
//
import javax.management.MBeanServer;
import com.sun.management.snmp.SnmpStatusException;

// jdmk imports
//
import com.sun.management.snmp.agent.SnmpMib;
import com.sun.management.snmp.agent.SnmpStandardObjectServer;
import com.sun.management.snmp.agent.SnmpStandardMetaServer;
import com.sun.management.snmp.SnmpDefinitions;

/**
 * The class is used for representing SNMP metadata for the "JmxMBeanServer" 
 * group.
 * It extends the generated JmxMBeanServerMeta class in order to override 
 * the table-meta factory methods in order to instantiate customized 
 * implementations of the JmxMBeanTableMeta (customized in
 * JmxMBeanTableMetaImpl) and JmxMBeanAttrTableMeta (customized in
 * JmxMBeanAttrTableMetaImpl) meta classes.
 */
public class JmxMBeanServerMetaImpl extends JmxMBeanServerMeta {

    /**
     * Constructor for the metadata associated to "JmxMBeanServer".
     */
    public JmxMBeanServerMetaImpl(SnmpMib myMib, 
				  SnmpStandardObjectServer objserv) {
	super(myMib,objserv);
    }


    /**
     * Factory method for "JmxMBeanAttrTable" table metadata class.
     * 
     * You can redefine this method if you need to replace the default
     * generated metadata class with your own customized class.
     * 
     * @param tableName Name of the table object ("JmxMBeanAttrTable")
     * @param groupName Name of the group to which this table belong 
     * ("JmxMBeanServer")
     * @param mib The SnmpMib object in which this table is registered
     * @param server MBeanServer for this table entries (may be null)
     * 
     * @return An instance of the metadata class generated for the
     *         "JmxMBeanAttrTable" table (JmxMBeanAttrTableMeta)
     * 
     **/
    protected JmxMBeanAttrTableMeta 
	createJmxMBeanAttrTableMetaNode(String tableName, String groupName, 
					SnmpMib mib, MBeanServer server)  {
        return new JmxMBeanAttrTableMetaImpl(mib, objectserver);
    }


    /**
     * Factory method for "JmxMBeanTable" table metadata class.
     * 
     * You can redefine this method if you need to replace the default
     * generated metadata class with your own customized class.
     * 
     * @param tableName Name of the table object ("JmxMBeanTable")
     * @param groupName Name of the group to which this table belong 
     * ("JmxMBeanServer")
     * @param mib The SnmpMib object in which this table is registered
     * @param server MBeanServer for this table entries (may be null)
     * 
     * @return An instance of the metadata class generated for the
     *         "JmxMBeanTable" table (JmxMBeanTableMeta)
     * 
     **/
    protected JmxMBeanTableMeta 
	createJmxMBeanTableMetaNode(String tableName, String groupName, 
				    SnmpMib mib, MBeanServer server)  {
        return new JmxMBeanTableMetaImpl(mib, objectserver);
    }
}
