/*
 * @(#)file      DEMO_MIB_IMPL.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.23
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

import javax.management.MBeanServer;
import javax.management.ObjectName;

/**
 * This class shows how a generated MIB class can be subclassed to
 * provide an implementation of the MIB.
 * <p>In particular, it shows how to override MBean factory methods
 *    in order to substitute the generated MBean skeletons with
 *    a specific implementation.</p>
 *
 * <p>The recommended steps to follow when developing a MIB are thus:
 * <ul>
 *     <li>For each group you want to provide an implementation of, 
 *         create a subclass of this group in which you will fill
 *         the get/set methods you need to override. (e.g.: for the "demo"
 *         group of DEMO-MIB, provide a DemoImpl class extending Demo
 *         in which the get/set method will be implemented).
 *     <li>Subclass the generated MIB (if not yet done), redefine
 *         the factory method for the group you want to implement, and
 *         make it return an instance of your implementation class in place
 *         of the generated "dummy" skeleton (e.g.: override the 
 *         <code>createDemoMBean()</code> factory method and make it
 *         return an instance of DemoImpl instead of an instance of Demo.
 * </ul>
 * <p>By proceeding this way you should never have to modify directly
 *    the generated files, which will greatly ease the development 
 *    process. </p>
 *
 **/
class DEMO_MIB_IMPL extends DEMO_MIB {

    public DEMO_MIB_IMPL() {
	super();
    }

    /**
     * Factory method for "Demo" group MBean.
     * 
     * This method is redefined in order to replace the instance of the 
     * generated System class with an instance of a customized class 
     * (DemoImpl) implementing the Demo group. 
     * 
     * @param groupName Name of the group ("Demo")
     * @param groupOid  OID of this group
     * @param groupObjname ObjectName for this group (may be null)
     * @param server    MBeanServer for this group (may be null)
     * 
     * @return An instance of the MBean class implementing the
     *         "Demo" group (DemoImpl)
     * 
     * Note that when using standard metadata,
     * the returned object must implement the "InterfacesMBean"
     * interface.
     **/
    protected Object createDemoMBean(String groupName, String groupOid, 
					ObjectName groupObjname, 
					MBeanServer server)  {

        // Note that when using standard metadata,
        // the returned object must implement the "InterfacesMBean"
        // interface.
        //
        if (server != null) 
            return new DemoImpl(this,server);
        else 
            return new DemoImpl(this);
    }

}
